//======================================================================
// File:		wvlogfile.cpp
// Author:	Matthias Toussaint
// Created:	Son Nov 12 18:17:48 CET 2000
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2000 Matthias Toussaint
//======================================================================

#include <wvlogfile.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qdatetime.h>

QString
WVLogfileEntry::logLine() const
{
  QString tmp;
  
  tmp.sprintf( "%s:%u:%u:%u:%u\n",
               acc.latin1(), startT, stopT, iB, oB );
  
  return tmp;
}

QString
WVLogfileEntry::day() const
{
  QString tmp;
  QDateTime dt;
  dt.setTime_t( startT );
  
  tmp.sprintf( "%s %d", 
               dt.date().dayName( dt.date().dayOfWeek() ).latin1(), 
               dt.date().day() );
  
  return tmp;
}

QString
WVLogfileEntry::startTime() const
{
  QString tmp;
  QDateTime dt;
  dt.setTime_t( startT );
  
  tmp.sprintf( "%02d:%02d:%02d", 
               dt.time().hour(), dt.time().minute(), dt.time().second() );
  
  return tmp;
}

QString
WVLogfileEntry::duration() const
{
  QString tmp;
  unsigned dur = stopT-startT;
  
  int h = dur/60/60;
  int m = (dur/60) % 60;
  int s = dur % 60;
  
  if (h)
  {
    tmp.sprintf( "%dh %dmin %ds", h, m, s );
  }
  else if (m)
  {
    tmp.sprintf( "%dmin %ds", m, s );
  }
  else
  {
    tmp.sprintf( "%ds", s );
  }
    
  return tmp;
}

QString
WVLogfileEntry::download() const
{
  char *unit[3] = { "bytes", "KB", "MB" };
  int   ui = 0;
  QString tmp;
  
  double D = (double)iB;
  
  while (D > 1024. && ui < 2)
  {
    D /= 1024.;
    ui++;
  }
  
  if (ui == 0)
  {
    tmp.sprintf( "%d bytes", iB );
  }
  else
  {
    tmp.sprintf( "%.2f %s", D, unit[ui] );
  } 
  
  return tmp;
}

QString
WVLogfileEntry::upload() const
{
  char *unit[3] = { "bytes", "KB", "MB" };
  int   ui = 0;
  QString tmp;
  
  double U = (double)oB;
  
  while (U > 1024. && ui < 2)
  {
    U /= 1024.;
    ui++;
  }
  
  if (ui == 0)
  {
    tmp.sprintf( "%d bytes", oB );
  }
  else
  {
    tmp.sprintf( "%.2f %s", U, unit[ui] );
  } 
  
  return tmp;
}

WVLogfile::WVLogfile()
{
  list.setAutoDelete( true );
}

WVLogfile::WVLogfile( const QString & fname ) :
  filename( fname )
{
  list.setAutoDelete( true );
}

WVLogfile::~WVLogfile()
{
}

void
WVLogfile::load()
{
  QFile file( filename );
  
  list.clear();
  
  if (file.open( IO_ReadOnly ))
  {
    char *buffer = new char [2000];
    
    while (file.readLine( buffer, 1999 ) > 0)
    {
      QString account = strtok( buffer, ":" );
      QString start = strtok( 0, ":" );
      QString stop = strtok( 0, ":" );
      QString ibytes = strtok( 0, ":" );
      QString obytes = strtok( 0, ":\n" );
      
      list.append( new WVLogfileEntry( account, start.toUInt(), stop.toUInt(),
                                       ibytes.toUInt(), obytes.toUInt() ));
    }
    
    delete [] buffer;
    
    file.close();
  }
}

void
WVLogfile::clear()
{
  list.clear();
}

void
WVLogfile::append( const QString & account, unsigned start,
                   unsigned stop, unsigned ibytes, unsigned obytes )
{
  WVLogfileEntry *entry = new WVLogfileEntry( account, start, stop,
                                              ibytes, obytes );
  
  QFile file( filename );
  
  if (file.open( IO_WriteOnly | IO_Append ))
  {
    QTextStream s( &file );
    
    s << entry->logLine();
    
    file.close();
  }
  
  list.append( entry );
}

void
WVLogfile::fillList( QListView * lv ) const
{
  lv->clear();
  lv->setUpdatesEnabled( false );
  
  QListIterator<WVLogfileEntry> it(list);
  WVLogfileEntry *entry;
  
  while ((entry = it.current()))
  {
    new QListViewItem( lv, 
                       entry->account(), entry->day(), entry->startTime(),
                       entry->duration(), entry->download(), entry->upload() );
    
    ++it;
  }
  
  lv->setUpdatesEnabled( true );
  lv->update();
}
